// TODO :
// - Base
// - API
// - UI
// - PkgDB Abstraction
// - Pkg Abstraction

use based::get_pg;
use based::page::{Shell, render_page};
use based::request::{RequestContext, StringResponse};
use maud::html;
use pkg::Repository;
use rocket::get;
use rocket::routes;

pub mod pkg;
pub mod routes;

#[get("/")]
pub async fn index_page(ctx: RequestContext) -> StringResponse {
    let repos: Vec<String> = Repository::list();

    let content = html!(
        h1 { "Repositories" };
        @for repo in repos {
            p { (repo) };
        };
    );

    render_page(
        content,
        "Repositories",
        ctx,
        &Shell::new(html! {}, html! {}, Some(String::new())),
    )
    .await
}

#[rocket::launch]
async fn launch() -> _ {
    env_logger::init();

    let pg = get_pg!();
    sqlx::migrate!("./migrations").run(pg).await.unwrap();

    rocket::build().mount("/", routes![
        index_page,
        routes::pkg_route,
        routes::upload_pkg
    ])
}
